package com.winhc.repal.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.winhc.repal.entity.RepalBill;
import com.winhc.repal.entity.RepalBillDiagnosis;
import com.winhc.repal.entity.RepalRemindDefinition;
import com.winhc.repal.entity.RepalRemindHistory;
import com.winhc.repal.enums.*;
import com.winhc.repal.model.vo.RemindTypeUnReadCountVO;
import com.winhc.repal.model.vo.RepalBillRemindUnReadVO;
import com.winhc.repal.repository.RepalRemindHistoryMapper;
import com.winhc.repal.service.RepalRemindDefinitionService;
import com.winhc.repal.service.RepalRemindHistoryService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.winhc.common.model.base.VOPage;
import com.winhc.repal.model.dto.RepalRemindHistoryDTO;
import com.winhc.repal.model.cvt.RepalRemindHistoryConvert;
import com.winhc.repal.model.vo.RepalRemindHistoryVO;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @description RepalRemindHistory 接口实现类
 * @author Generator
 * @date 2022-04-08
 */
@Service
public class RepalRemindHistoryServiceImpl extends ServiceImpl<RepalRemindHistoryMapper, RepalRemindHistory> implements RepalRemindHistoryService {
    @Autowired
    private RepalRemindHistoryMapper repalRemindHistoryMapper;
    @Autowired
    private RepalRemindDefinitionService repalRemindDefinitionService;

    @Override
    public RepalBillRemindUnReadVO getRemindUnReadVO(List<Long> repalBillIds) {
        RepalBillRemindUnReadVO result = new RepalBillRemindUnReadVO(0,0,0,0,0);
        List<RemindTypeUnReadCountVO> unReadCounts = repalRemindHistoryMapper.getRemindUnReadVO(repalBillIds);
        if (CollUtil.isEmpty(unReadCounts)) {
            // 没有未读
            return result;
        }
        unReadCounts.forEach(countVO -> {
            if (RemindEnum.风险预警.name().equals(countVO.getRemindType())) {
                result.setUnreadRiskCount(countVO.getCount());
            }
            if (RemindEnum.财产线索.name().equals(countVO.getRemindType())) {
                result.setUnreadFinanceCount(countVO.getCount());
            }
            if (RemindEnum.账款评级.name().equals(countVO.getRemindType())) {
                result.setUnreadRankCount(countVO.getCount());
            }
            if (RemindEnum.逾期提醒.name().equals(countVO.getRemindType())) {
                result.setUnreadOverdueCount(countVO.getCount());
            }
        });
        result.setUnreadTotalCount(unReadCounts.stream().mapToInt(RemindTypeUnReadCountVO::getCount).sum());
        return result;
    }

    @Override
    public boolean insertRankRemind(RepalBill repalBill, RepalBillDiagnosis diagnosis) {
        if (repalBill.getRepalBillStage() == null) {
            return true;
        }
        LocalDate endDate = repalBill.getEndDate();
        if (endDate == null) {
            endDate = LocalDate.now().minusDays(1);
        }
        OverdueEnum overdueEnum = getDaysBetween(endDate);
        LambdaQueryWrapper<RepalRemindDefinition> qw = Wrappers.lambdaQuery(RepalRemindDefinition.class);
        qw.eq(RepalRemindDefinition::getRemindType, RemindEnum.账款评级.name());
        if (RepalBillStatusEnum.PROGRESSING.getCode().equals(repalBill.getRepalBillStatus())
                || !RepalBillStageEnum.NON_PROSECUTE.getCode().equals(repalBill.getRepalBillStage())) {
            qw.eq(RepalRemindDefinition::getOpportunity, diagnosis.getDiagnosisGrade());
        } else {
            qw.eq(RepalRemindDefinition::getOpportunity, diagnosis.getDiagnosisGrade() + "+" + overdueEnum.getCode());
        }
        qw.eq(RepalRemindDefinition::getRepalBillStatus, repalBill.getRepalBillStatus());
        qw.eq(RepalRemindDefinition::getRepalBillStage, repalBill.getRepalBillStage());
        RepalRemindDefinition accountRemindDefinition = repalRemindDefinitionService.getOne(qw, false);
        if (accountRemindDefinition != null) {
            String changeStr = "下降";
            String changeStr1 = "下降";
            if (repalBill.getDiagnosisScore().compareTo(repalBill.getPreDiagnosisScore() == null ? new BigDecimal(0)
                    : repalBill.getPreDiagnosisScore()) > 0) {
                changeStr = "上升";
                changeStr1 = "提高";
            }
            RepalRemindHistory accountRemindHistory = new RepalRemindHistory();
            accountRemindHistory.setUserId(repalBill.getUserId())
                    .setRepalBookId(repalBill.getRepalBookId())
                    .setRepalBillId(repalBill.getId())
                    .setTrendId(diagnosis.getId().toString())
                    .setTrendContent(repalBill.getPreDiagnosisId() != null ? repalBill.getPreDiagnosisId().toString() : null)
                    .setRemindTime(LocalDateTime.now())
                    .setCompanyName(repalBill.getCustomerName())
                    .setRemindTitle(diagnosis.getDiagnosisGrade() + "（" + diagnosis.getDiagnosisScore().stripTrailingZeros().toPlainString() + "分）")
                    .setRemindContent(StrUtil.format(accountRemindDefinition.getRemindContent(),
                            changeStr,
                            changeStr1,
                            overdueEnum.name()))
                    .setFunctionList(accountRemindDefinition.getFunctionList())
                    .setRemindType(RemindEnum.账款评级.name())
                    .setReadStatus(ReadStatusEnum.未读.getCode());
            save(accountRemindHistory);
//            //推送
//            AccountRemindSettingVO accountRemindSetting = accountRemindSettingService.find(accountBill.getUserId());
//            String redisKey = MESSAGE_PRIFIX + repalBill.getUserId() + "_" + Dict.RemindEnum.账款评级.name();
//            if (Dict.REMIND_SETTING_ENUM.需要提醒.getCode().equals(accountRemindSetting.getRank()) &&
//                    redisUtil.get(redisKey) == null && accountBill.getPreDiagnosisId() != null) {
//                Map<String, String> paraMap = new HashMap<>();
//                paraMap.put("company", repalBill.getAccountBillTitle());
//                paraMap.put("pregrade", repalBill.getPreDiagnosisGrade());
//                paraMap.put("prescore", repalBill.getPreDiagnosisScore().toPlainString());
//                paraMap.put("change", changeStr);
//                paraMap.put("grade", repalBill.getDiagnosisGrade());
//                paraMap.put("score", repalBill.getDiagnosisScore().toPlainString());
//                paraMap.put("change1", changeStr1);
//                paraMap.put("wizardId", repalBill.getId().toString());
//                paraMap.put("accountBookId", repalBill.getAccountBookId().toString());
//                messageNoticeService.sendPushmsg(repalBill.getUserId(), Dict.MsgNameEnum.评级变动APP提醒, paraMap);
//                messageNoticeService.sendSms(repalBill.getUserId(), Dict.MsgNameEnum.评级变动短信提醒, paraMap);
//                redisUtil.setEx(redisKey, "done", calTTL(), TimeUnit.MILLISECONDS);
//            }


            //掐头去尾压进accountbill.remindInfo中
            String remindContent = accountRemindHistory.getRemindContent();
            if (remindContent.length() > 100) {
                remindContent = remindContent.substring(0, 100);
            }
            accountRemindHistory.setFunctionList(null).setRemindContent(remindContent);
            repalBill.setRemindInfo(JSON.toJSONString(accountRemindHistory));
        }
        return true;
    }

    private OverdueEnum getDaysBetween(LocalDate endDate) {
        long delta = LocalDate.now().toEpochDay() - endDate.toEpochDay();
        OverdueEnum overdueEnum = OverdueEnum.其他;
        if (delta <= 0 && delta >= -7) {
            overdueEnum = OverdueEnum.即将逾期;
        } else if (delta > 0 && delta < 90) {
            overdueEnum = OverdueEnum.已逾期;
        } else if (delta >= 90 && delta < 180) {
            overdueEnum = OverdueEnum.已逾期三个月;
        } else if (delta >= 180 && delta < 365) {
            overdueEnum = OverdueEnum.已逾期半年;
        } else if (delta >= 365 && delta < 365 * 3) {
            overdueEnum = OverdueEnum.已逾期一年;
        } else if (delta >= 365 * 3) {
            overdueEnum = OverdueEnum.已逾期三年;
        }
        return overdueEnum;
    }
}
